# -*- coding: utf-8 -*-
from django.conf import settings
from django.core.mail.backends.base import BaseEmailBackend

from .tasks import send_emails


class CeleryEmailBackend(BaseEmailBackend):
    def __init__(self, fail_silently=False, **kwargs):
        super(CeleryEmailBackend, self).__init__(fail_silently)
        self.init_kwargs = kwargs

    def send_messages(self, email_messages, **kwargs):
        if not email_messages:
            return 0

        kwargs["_backend_init_kwargs"] = self.init_kwargs

        NO_DELAY = getattr(settings, "DJCELERY_SES_NO_DELAY", False)
        CHUNK_SIZE = getattr(settings, "DJCELERY_SES_CHUNK_SIZE", 50)

        for index in range(0, len(email_messages), CHUNK_SIZE):
            email_messages_chunk = email_messages[index: index + CHUNK_SIZE]

            if NO_DELAY:
                send_emails(email_messages_chunk, **kwargs)
            else:
                send_emails.delay(email_messages_chunk, **kwargs)

        return len(email_messages)



# coding: utf-8
import json
import re

from django.http import HttpResponse, HttpResponseBadRequest
from django.views.decorators.csrf import csrf_exempt
from django.core.mail import mail_admins
from django.core.validators import validate_email
from django.core.exceptions import ValidationError

from .models import Blacklist


@csrf_exempt
def sns_notification(request):
    """
    Receive AWS SES bounce SNS notification
    """

    # decode json
    try:
        data = json.loads(request.read())
    except ValueError:
        return HttpResponseBadRequest('Invalid JSON')

    # handle SNS subscription
    if data['Type'] == 'SubscriptionConfirmation':
        subscribe_url = data['SubscribeURL']
        subscribe_body = """
        Please visit this URL below to confirm your subscription with SNS

        %s """ % subscribe_url

        mail_admins('Please confirm SNS subscription', subscribe_body)
        return HttpResponse('OK')

    try:
        message = json.loads(data['Message'])
    except ValueError:
        assert False, data['Message']

    notification_type = message['notificationType']
    if notification_type not in dict(Blacklist.TYPE_CHOICES).values():
        return HttpResponse('No Email')

    type = 0 if notification_type == 'Bounce' else 1
    email = message['mail']['destination'][0]

    try:
        validate_email(email)
    except ValidationError:
        try:
            email = re.findall(r"<(.+?)>", email)[0]
        except IndexError:
            email = None

    if not email:
        return HttpResponse('Email Error')

    # add email to blacklist
    Blacklist.objects.get_or_create(email=email, defaults={"type": type})

    return HttpResponse('Done')



from smtplib import SMTPDataError

from django.conf import settings
from django.db import IntegrityError
from django.core.mail import get_connection

from celery.task import task

from .models import Blacklist, MessageLog


CONFIG = getattr(settings, 'CELERY_EMAIL_TASK_CONFIG', {})
BACKEND = getattr(settings, 'CELERY_EMAIL_BACKEND',
                  'django.core.mail.backends.smtp.EmailBackend')

TASK_CONFIG = {
    'name': 'djcelery_email_send',
    'ignore_result': True,
}
TASK_CONFIG.update(CONFIG)


@task(**TASK_CONFIG)
def send_emails(messages, **kwargs):
    """
    send mails task
    """
    logger = send_emails.get_logger()
    conn = get_connection(backend=BACKEND)
    conn.open()

    num = 0
    for message in messages:
        # check blacklist
        CHECK_BLACKLIST = getattr(
            settings, 'DJCELERY_SES_CHECK_BLACKLIST', True)
        if CHECK_BLACKLIST:
            logger.debug('Check blacklist')

            try:
                Blacklist.objects.get(email=message.to[0], type=0)
                logger.debug("Email already in blacklist.")
                continue
            except Blacklist.DoesNotExist:
                pass

        # send
        try:
            result = conn.send_messages([message])
            logger.debug("Successfully sent email message to %r.", message.to)
            MessageLog.objects.log(message, 1)
            num += result
        except SMTPDataError as e:
            logger.warning("Message to %r, blacklisted.", message.to)
            if e.smtp_code == 554:
                MessageLog.objects.log(message, 3)
                try:
                    Blacklist(email=message.to[0]).save()
                except IntegrityError:
                    pass
        except Exception as e:
            MessageLog.objects.log(message, 2)
            logger.warning(
                "Failed to send email message to %r, retrying.", message.to)
            if len(messages) == 1:
                send_emails.retry(exc=e)
            else:
                send_emails.delay([message], **kwargs)

    conn.close()
    return num
